@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004-2016 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2004-2017 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -20,12 +20,9 @@
 #ifndef KDB_H
 #define KDB_H
 
-#include <QList>
-#include <QVariant>
-#include <QByteArray>
-
-#include "KDbGlobal.h"
 #include "KDbField.h"
+#include "KDbGlobal.h"
+#include "KDbSqlResult.h"
 #include "KDbTableSchema.h"
 
 class QDomNode;
@@ -38,10 +35,8 @@ class KDbDriver;
 class KDbEscapedString;
 class KDbLookupFieldSchema;
 class KDbMessageHandler;
-class KDbQuerySchema;
 class KDbResultable;
 class KDbResultInfo;
-class KDbSqlResult;
 class KDbTableOrQuerySchema;
 class KDbVersionInfo;
 class tristate;
@@ -53,39 +48,43 @@ namespace KDb
 //! @see KDbConnection::databaseVersion() KDbConnection::serverVersion() KDbDriverMetaData::version()
 KDB_EXPORT KDbVersionInfo version();
 
-//! @overload bool deleteRecord(KDbConnection*, const KDbTableSchema&, const QString &, KDbField::Type, const QVariant &)
+/**
+ * @brief Deletes records using one generic criteria.
+ *
+ * @return @c true on success and @c false on failure and if @a conn is @c nullptr
+ */
 KDB_EXPORT bool deleteRecords(KDbConnection* conn, const QString &tableName,
                               const QString &keyname, KDbField::Type keytype, const QVariant &keyval);
 
-//! Deletes records using one generic criteria.
+//! @overload
 inline bool deleteRecords(KDbConnection* conn, const KDbTableSchema &table,
                               const QString &keyname, KDbField::Type keytype, const QVariant &keyval)
 {
     return deleteRecords(conn, table.name(), keyname, keytype, keyval);
 }
 
-//! @overload bool deleteRecords(KDbConnection*, const QString&, const QString&, KDbField::Type, const QVariant&);
+//! @overload
 inline bool deleteRecords(KDbConnection* conn, const QString &tableName,
                               const QString &keyname, const QString &keyval)
 {
     return deleteRecords(conn, tableName, keyname, KDbField::Text, keyval);
 }
 
-//! @overload bool deleteRecords(KDbConnection*, const QString&, const QString&, const QString&);
+//! @overload
 inline bool deleteRecords(KDbConnection* conn, const KDbTableSchema &table,
                               const QString &keyname, const QString &keyval)
 {
     return deleteRecords(conn, table.name(), keyname, keyval);
 }
 
-//! @overload bool deleteRecords(KDbConnection*, const KDbTableSchema&, const QString&, const QString&);
+//! @overload
 inline bool deleteRecords(KDbConnection* conn, const KDbTableSchema &table,
                               const QString& keyname, int keyval)
 {
     return deleteRecords(conn, table, keyname, KDbField::Integer, keyval);
 }
 
-//! @overload bool deleteRecords(KDbConnection*, const KDbTableSchema&, const QString&, int);
+//! @overload
 inline bool deleteRecords(KDbConnection* conn, const QString &tableName,
                               const QString& keyname, int keyval)
 {
@@ -112,32 +111,39 @@ inline bool deleteAllRecords(KDbConnection* conn, const KDbTableSchema &table)
     return KDb::deleteAllRecords(conn, table.name());
 }
 
-/*! @return autoincrement field's @a autoIncrementFieldName value
- of last inserted record for result @a result.
+/**
+ * Returns value of last inserted record for an autoincrement field
+ *
+ * This method internally fetches values of the last inserted record for which @a result was
+ * returned and returns selected field's value. The field belong to @a tableName table.
+ * The field must be of integer type, there must be a record inserted using query for which the @a
+ * result is returned.
+ * std::numeric_limits<quint64>::max() is returned on error or if @a result is null.
+ * The last inserted record is identified by a magical record identifier, usually called ROWID
+ * (PostgreSQL has it as well as SQLite;
+ * see KDbDriverBehavior::ROW_ID_FIELD_RETURNS_LAST_AUTOINCREMENTED_VALUE).
+ * ROWID's value will be assigned back to @a recordId if this pointer is not @c nullptr.
+ */
+KDB_EXPORT quint64 lastInsertedAutoIncValue(QSharedPointer<KDbSqlResult> result,
+                                            const QString &autoIncrementFieldName,
+                                            const QString &tableName, quint64 *recordId = nullptr);
 
- Simply, method internally fetches values of the last inserted record and returns selected
- field's value. The field belong to @a tableName table.
- Requirements: the field must be of integer type, there must be a record inserted using query
- for which the @a result is returned.
- On error std::numeric_limits<quint64>::max() is returned.
- Last inserted record is identified by a magical record identifier, usually called ROWID
- (PostgreSQL has it as well as SQLite; see KDbDriverBehavior::ROW_ID_FIELD_RETURNS_LAST_AUTOINCREMENTED_VALUE).
- ROWID's value will be assigned back to @a recordId if this pointer is not null.
-*/
-KDB_EXPORT quint64 lastInsertedAutoIncValue(KDbSqlResult *result,
-    const QString& autoIncrementFieldName, const QString& tableName, quint64* recordId = 0);
-
-/*! @overload int lastInsertedAutoIncValue(KDbSqlResult *, const QString&, const QString&, quint64*)
+/**
+ * @overload
+ *
  * Accepts @a recordId that can be obtained from KDbPreparedStatement::lastInsertRecordId()
  * or KDbSqlResult::lastInsertRecordId().
 */
 KDB_EXPORT quint64 lastInsertedAutoIncValue(KDbConnection *conn, const quint64 recordId,
-    const QString& autoIncrementFieldName, const QString& tableName);
+                                            const QString &autoIncrementFieldName,
+                                            const QString &tableName);
 
-/*! @overload int lastInsertedAutoIncValue(KDbSqlResult *, const QString&, const QString&, quint64*)
+/**
+@overload
 */
-inline quint64 lastInsertedAutoIncValue(KDbSqlResult *result,
-    const QString& autoIncrementFieldName, const KDbTableSchema& table, quint64* recordId = 0)
+inline quint64 lastInsertedAutoIncValue(QSharedPointer<KDbSqlResult> result,
+                                        const QString &autoIncrementFieldName,
+                                        const KDbTableSchema &table, quint64 *recordId = nullptr)
 {
     return lastInsertedAutoIncValue(result, autoIncrementFieldName, table.name(), recordId);
 }
@@ -173,76 +179,68 @@ KDB_EXPORT QString simplifiedFieldTypeName(KDbField::Type type);
  @see KDbField::hasEmptyProperty() */
 KDB_EXPORT bool isEmptyValue(KDbField::Type type, const QVariant &value);
 
-/*! Sets string pointed by @a msg to an error message retrieved from @a resultable,
- and string pointed by @a details to details of this error (server message and result number).
- Does nothing if @a result is empty. In this case @a msg and @a details strings are not overwritten.
- If the string pointed by @a msg is not empty, @a result message is appended to the string
- pointed by @a details.
+/**
+ * @brief Sets HTML-formatted error message with extra details obtained from result object
+ *
+ * Sets string pointed by @a msg to an error message retrieved from @a resultable,
+ * and string pointed by @a details to details of this error (server message and result number).
+ * Does nothing if there is no error (resultable.result().isError() == false) or if @a msg or
+ * @a details is @c nullptr.
+ * In this case strings pointer by @a msg and @a details strings are not changed.
+ * If the string pointed by @a msg is not empty, it is not modified and message obtained from
+ * @a resultable is appended to the string pointed by @a details instead.
  */
 KDB_EXPORT void getHTMLErrorMesage(const KDbResultable& resultable, QString *msg, QString *details);
 
-/*! This methods works like above, but appends both a message and a description
- to string pointed by @a msg. */
+/**
+ * @overload
+ *
+ * This methods works similarly but appends both a message and a description to string pointed by
+ * @a msg.
+ */
 KDB_EXPORT void getHTMLErrorMesage(const KDbResultable& resultable, QString *msg);
 
-/*! This methods works like above, but works on @a result's  members instead. */
+/**
+ * @overload
+ *
+ * This methods similarly but outputs message to @a info instead.
+ */
 KDB_EXPORT void getHTMLErrorMesage(const KDbResultable& resultable, KDbResultInfo *info);
 
 /*! Function useful for building WHERE parts of SQL statements.
  Constructs an SQL string like "fielname = value" for specific @a drv driver,
- field type @a t, @a fieldName and @a value. If @a value is null, "fieldname is NULL"
+ field type @a t, @a fieldName and @a value. If @a value is null, "fieldname IS NULL"
  string is returned. */
 KDB_EXPORT KDbEscapedString sqlWhere(KDbDriver *drv, KDbField::Type t,
                                         const QString& fieldName, const QVariant& value);
 
-/*! Find an identifier for object @a objName of type @a objType.
- On success true is returned and *id is set to the value of the identifier.
- On failure false is returned. If there is no such object, @c cancelled value is returned. */
+/**
+ * @brief Finds an identifier for object @a objName of type @a objType
+ *
+ * On success true is returned and *id is set to the value of the identifier.
+ * On failure or if @a conn is @c nullptr, @c false is returned.
+ * If there is no object with specified name and type, @c cancelled value is returned.
+*/
 KDB_EXPORT tristate idForObjectName(KDbConnection* conn, int *id, const QString& objName,
                                     int objType);
 
-//! @todo perhaps use quint64 here?
-/*! @return number of records that can be retrieved after executing @a sql statement
- within a connection @a conn. The statement should be of type SELECT.
- For SQL data sources it does not fetch any records, only "COUNT(*)"
- SQL aggregation is used at the backed.
- -1 is returned if error occurred. */
-KDB_EXPORT int recordCount(KDbConnection* conn, const KDbEscapedString& sql);
-
-//! @todo perhaps use quint64 here?
-/*! @return number of records that can be retrieved from @a tableSchema.
- The table must be created or retrieved using a KDbConnection object,
- i.e. tableSchema.connection() must not return 0.
- For SQL data sources it does not fetch any records, only "COUNT(*)"
- SQL aggregation is used at the backed.
- -1 is returned if error occurred. */
-KDB_EXPORT int recordCount(const KDbTableSchema& tableSchema);
-
-/*! @overload in rowCount(const KDbTableSchema& tableSchema)
- Operates on a query schema. @a params are optional values of parameters that will
- be inserted into places marked with [] before execution of the query. */
-//! @todo perhaps use quint64 here?
-KDB_EXPORT int recordCount(KDbQuerySchema* querySchema,
-                           const QList<QVariant>& params = QList<QVariant>());
-
-/*! @overload int rowCount(KDbQuerySchema& querySchema, const QList<QVariant>& params)
- Operates on a table or query schema. @a params are optional values of parameters that
- will be inserted into places marked with [] before execution of the query. */
-//! @todo perhaps use quint64 here?
-KDB_EXPORT int recordCount(KDbTableOrQuerySchema* tableOrQuery,
-                           const QList<QVariant>& params = QList<QVariant>());
-
-/*! @return a number of columns that can be retrieved from table or query schema.
- In case of query, expanded fields are counted. Can return -1 if @a tableOrQuery
- has neither table or query assigned. */
-KDB_EXPORT int fieldCount(KDbTableOrQuerySchema* tableOrQuery);
-
-/*! shows connection test dialog with a progress bar indicating connection testing
- (within a second thread).
- @a data is used to perform a (temporary) test connection. @a msgHandler is used to display errors.
- On successful connecting, a message is displayed. After testing, temporary connection is closed. */
-KDB_EXPORT void connectionTestDialog(QWidget* parent, const KDbConnectionData& data,
-        KDbMessageHandler* msgHandler);
+/**
+ * @brief Shows connection test dialog
+ *
+ * Shows connection test dialog with a progress bar indicating connection testing
+ * (within a separate thread). @a data is used to perform a (temporary) test connection.
+ * @a msgHandler can be used for error handling. @a parent is used as dialog's parent widget.
+ *
+ * The dialog is modal so the call is blocking.
+ *
+ * On successful connecting, a successfull message of type KDbMessageHandler::Information is passed
+ * to @a msgHandler. After testing, temporary connection is closed.
+ *
+ * @return @c true for successfull connecting, @c for failed connecting and @c cancelled if the test
+ * has been cancelled.
+ */
+KDB_EXPORT tristate showConnectionTestDialog(QWidget* parent, const KDbConnectionData& data,
+                                             KDbMessageHandler* msgHandler);
 
 //! Used in splitToTableAndFieldParts().
 enum SplitToTableAndFieldPartsOptions {
@@ -258,6 +256,7 @@ enum SplitToTableAndFieldPartsOptions {
     (the default), or
  - '.' character is the first of last character of @a string (in this case table name
    or field name could become empty what is not allowed).
+ - @a tableName or @a fieldName is @c nullptr
 
  If @a option is SetFieldNameIfNoTableName and @a string does not contain '.',
  @a string is passed to @a fieldName and @a tableName is set to QString()
@@ -290,11 +289,21 @@ KDB_EXPORT bool supportsVisibleDecimalPlacesProperty(KDbField::Type type);
  @see KDb::numberToLocaleString() KDbField::visibleDecimalPlaces() */
 KDB_EXPORT QString numberToString(double value, int decimalPlaces);
 
-/*! Like KDb::numberToString() but formats the string using locale.toString().
- If @a locale if @c nullptr, desault QLocale is used.
+/**
+ * @brief Returns number converted to string using default locale
+ *
+ * This method is similar to KDb::numberToString() but the string is formatted using QLocale::toString().
+ *
+ * @see KDb::numberToString() KDbField::visibleDecimalPlaces()
+ */
+KDB_EXPORT QString numberToLocaleString(double value, int decimalPlaces);
 
- @see KDb::numberToString() KDbField::visibleDecimalPlaces() */
-KDB_EXPORT QString numberToLocaleString(double value, int decimalPlaces, const QLocale *locale = nullptr);
+/**
+ * @overload
+ *
+ * Returns number converted to string using specified locale.
+ */
+KDB_EXPORT QString numberToLocaleString(double value, int decimalPlaces, const QLocale &locale);
 
 //! @return true if @a propertyName is a builtin field property.
 KDB_EXPORT bool isBuiltinTableFieldProperty(const QByteArray& propertyName);
@@ -319,13 +328,17 @@ KDB_EXPORT KDbField::Type intToFieldType(int type);
 KDB_EXPORT KDbField::TypeGroup intToFieldTypeGroup(int typeGroup);
 
 /*! Gets property values for the lookup schema @a lookup.
- @a values is cleared before filling. This function is used e.g. for altering table design. */
+ @a values is not cleared before filling. This function is used e.g. for altering table design.
+ Nothing is performed if @a values is @c nullptr.
+ If @a lookup is @c nullptr, all returned values are null.
+*/
 KDB_EXPORT void getProperties(const KDbLookupFieldSchema *lookup, QMap<QByteArray, QVariant> *values);
 
 /*! Gets property values for @a field.
  Properties from extended schema are included. @a values is cleared before filling.
  The same number of properties in the same order is returned.
  This function is used e.g. for altering table design.
+ Nothing is performed if @a values is @c nullptr.
  */
 KDB_EXPORT void getFieldProperties(const KDbField &field, QMap<QByteArray, QVariant> *values);
 
@@ -333,23 +346,25 @@ KDB_EXPORT void getFieldProperties(const KDbField &field, QMap<QByteArray, QVari
  On failure contents of @a field is undefined.
  Properties from extended schema are also supported.
  This function is used e.g. by KDbAlterTableHandler when property information comes in form of text.
+ If @a field is @c nullptr nothing is performed and @c false is returned.
  */
 KDB_EXPORT bool setFieldProperties(KDbField *field, const QMap<QByteArray, QVariant>& values);
 
-/*! Sets property value for @a field. @return true if the property has been found and
- the value is valid for this property. On failure contents of @a field is undefined.
- Properties from extended schema are also supported as well as
-   QVariant customProperty(const QString& propertyName) const;
+/*! Sets value of a single property for @a field. @return true if the property has been found and
+ the value is valid for this property. On failure contents of @a field is not modified.
+ Properties from extended schema are also supported as well as custom properties
+ (using KDbField::setCustomProperty()).
 
  This function is used e.g. by KDbAlterTableHandler when property information comes in form of text.
+ If @a field is @c nullptr nothing is performed and @c false is returned.
  */
-KDB_EXPORT bool setFieldProperty(KDbField *field, const QByteArray& propertyName,
-                                       const QVariant& value);
+KDB_EXPORT bool setFieldProperty(KDbField *field, const QByteArray &propertyName,
+                                 const QVariant &value);
 
 /*! @return property value loaded from a DOM @a node, written in a QtDesigner-like
  notation: &lt;number&gt;int&lt;/number&gt; or &lt;bool&gt;bool&lt;/bool&gt;, etc. Supported types are
  "string", "cstring", "bool", "number". For invalid values null QVariant is returned.
- You can check the validity of the returned value using QVariant::type(). */
+ Validity of the returned value can be checked using the @a ok parameter and QVariant::type(). */
 KDB_EXPORT QVariant loadPropertyValueFromDom(const QDomNode& node, bool *ok);
 
 /*! Convenience version of loadPropertyValueFromDom(). @return int value. */
@@ -358,20 +373,30 @@ KDB_EXPORT int loadIntPropertyValueFromDom(const QDomNode& node, bool* ok);
 /*! Convenience version of loadPropertyValueFromDom(). @return QString value. */
 KDB_EXPORT QString loadStringPropertyValueFromDom(const QDomNode& node, bool* ok);
 
-/*! Saves integer element for value @a value to @a doc document within parent element
- @a parentEl. The value will be enclosed in "number" element and "elementName" element.
- Example: saveNumberElementToDom(doc, parentEl, "height", 15) will create
+/*! Creates a new DOM element named @a elementName with numeric value @a value in @a doc document
+ within parent element @a parentEl. The value will be enclosed in "number" element and
+ "elementName" element.
+ Example: saveNumberElementToDom(doc, parentEl, "height", 15) creates:
  @code
   <height><number>15</number></height>
  @endcode
- @return the reference to element created with tag elementName. */
+ @return the reference to element created with tag elementName.
+ Null element is returned if @a doc or @a parentEl is @c nullptr or if @a elementName is empty.
+*/
 KDB_EXPORT QDomElement saveNumberElementToDom(QDomDocument *doc, QDomElement *parentEl,
-        const QString& elementName, int value);
+                                              const QString& elementName, int value);
 
-/*! Saves boolean element for value @a value to @a doc document within parent element
- @a parentEl. Like saveNumberElementToDom() but creates "bool" tags. True/false values will be
+/*! Creates a new DOM element named @a elementName with boolean value @a value in @a doc document
+ within parent element @a parentEl.
+ This method is like saveNumberElementToDom() but creates "bool" tags. True/false values will be
  saved as "true"/"false" strings.
- @return the reference to element created with tag elementName. */
+ Example: saveBooleanElementToDom(doc, parentEl, "visible", true) creates:
+ @code
+  <visible><bool>true</bool></visible>
+ @endcode
+ @return the reference to element created with tag elementName.
+ Null element is returned if @a doc or @a parentEl is @c nullptr or if @a elementName is empty.
+*/
 KDB_EXPORT QDomElement saveBooleanElementToDom(QDomDocument *doc, QDomElement *parentEl,
         const QString& elementName, bool value);
 
@@ -397,7 +422,7 @@ KDB_EXPORT QVariant notEmptyValueForFieldType(KDbField::Type type);
 /*! @return true if the @a word is an reserved KDbSQL keyword
  See generated/sqlkeywords.cpp.
  @todo add function returning list of keywords. */
-KDB_EXPORT bool isKDbSQLKeyword(const QByteArray& word);
+KDB_EXPORT bool isKDbSqlKeyword(const QByteArray& word);
 
 //! @return @a string string with applied KDbSQL identifier escaping
 /*! This escaping can be used for field, table, database names, etc.
@@ -425,6 +450,26 @@ KDB_EXPORT QByteArray escapeIdentifierAndAddQuotes(const QByteArray& string);
     @see unescapeString() */
 KDB_EXPORT QString escapeString(const QString& string);
 
+/**
+ * @brief Returns escaped string @a string
+ *
+ * If @a drv driver is present, it is used to perform escaping, otherwise escapeString() is used
+ * so the KDbSQL dialect-escaping is performed.
+ *
+ * @since 3.1.0
+ */
+KDB_EXPORT KDbEscapedString escapeString(KDbDriver *drv, const QString& string);
+
+/**
+ * @brief Returns escaped string @a string
+ *
+ * If @a conn is present, its driver is used to perform escaping, otherwise escapeString() is used
+ * so the KDbSQL dialect-escaping is performed.
+ *
+ * @since 3.1.0
+ */
+KDB_EXPORT KDbEscapedString escapeString(KDbConnection *conn, const QString& string);
+
 //! Unescapes characters in string @a string for the KDbSQL dialect.
 /** The operation depends on @a quote character, which can be be ' or ".
  * @a string is assumed to be properly constructed. This is assured by the lexer's grammar.
@@ -487,14 +532,14 @@ KDB_EXPORT QString escapeString(const QString& string);
 KDB_EXPORT QString unescapeString(const QString& string, char quote, int *errorPosition = nullptr);
 
 //! Escaping types for BLOBS. Used in escapeBLOB().
-enum BLOBEscapingType {
-    BLOBEscapeXHex = 1,        //!< Escaping like X'1FAD', used by sqlite (hex numbers)
-    BLOBEscape0xHex,           //!< Escaping like 0x1FAD, used by mysql (hex numbers)
-    BLOBEscapeHex,             //!< Escaping like 1FAD without quotes or prefixes
-    BLOBEscapeOctal,           //!< Escaping like 'zk\\000$x', used by PostgreSQL
+enum class BLOBEscapingType {
+    XHex = 1,        //!< Escaping like X'1FAD', used by sqlite (hex numbers)
+    ZeroXHex,           //!< Escaping like 0x1FAD, used by mysql (hex numbers)
+    Hex,             //!< Escaping like 1FAD without quotes or prefixes
+    Octal,           //!< Escaping like 'zk\\000$x', used by PostgreSQL
                                //!< (only non-printable characters are escaped using octal numbers);
                                //!< see http://www.postgresql.org/docs/9.5/interactive/datatype-binary.html
-    BLOBEscapeByteaHex         //!< "bytea hex" escaping, e.g. E'\xDEADBEEF'::bytea used by PostgreSQL
+    ByteaHex         //!< "bytea hex" escaping, e.g. E'\xDEADBEEF'::bytea used by PostgreSQL
                                //!< (only non-printable characters are escaped using octal numbers);
                                //!< see http://www.postgresql.org/docs/9.5/interactive/datatype-binary.html
 };
@@ -520,8 +565,8 @@ KDB_EXPORT QByteArray pgsqlByteaToByteArray(const char* data, int length = -1);
  @a data is escaped in format X'*', where * is one or more hexadecimal digits.
  Both A-F and a-f letters are supported. Even and odd number of digits are supported.
  If @a ok is not 0, *ok is set to result of the conversion.
- See BLOBEscapeXHex. */
-KDB_EXPORT QByteArray xHexToByteArray(const char* data, int length = -1, bool *ok = 0);
+ See BLOBEscapingType::XHex. */
+KDB_EXPORT QByteArray xHexToByteArray(const char* data, int length = -1, bool *ok = nullptr);
 
 /*! @return byte array converted from @a data of length @a length.
  If @a length is negative, the data is assumed to point to a null-terminated string
@@ -529,12 +574,12 @@ KDB_EXPORT QByteArray xHexToByteArray(const char* data, int length = -1, bool *o
  @a data is escaped in format 0x*, where * is one or more hexadecimal digits.
  Both A-F and a-f letters are supported. Even and odd number of digits are supported.
  If @a ok is not 0, *ok is set to result of the conversion.
- See BLOBEscape0xHex. */
-KDB_EXPORT QByteArray zeroXHexToByteArray(const char* data, int length = -1, bool *ok = 0);
+ See BLOBEscapingType::ZeroXHex. */
+KDB_EXPORT QByteArray zeroXHexToByteArray(const char* data, int length = -1, bool *ok = nullptr);
 
 /*! @return int list converted from string list.
    If @a ok is not 0, *ok is set to result of the conversion. */
-KDB_EXPORT QList<int> stringListToIntList(const QStringList &list, bool *ok = 0);
+KDB_EXPORT QList<int> stringListToIntList(const QStringList &list, bool *ok = nullptr);
 
 /*! @return string converted from list @a list.
    Separators are ',' characters, "," and "\\" are escaped.
@@ -613,16 +658,16 @@ KDB_EXPORT QString defaultFileBasedDriverId();
 
 /*! Escapes and converts value @a v (for type @a ftype)
     to string representation required by KDbSQL commands.
-    For Date/Time type KDb::dateTimeToSQL() is used.
-    For BLOB type KDb::escapeBlob() with BLOBEscape0xHex conversion type is used. */
-KDB_EXPORT KDbEscapedString valueToSQL(KDbField::Type ftype, const QVariant& v);
+    For Date/Time type KDb::dateTimeToSql() is used.
+    For BLOB type KDb::escapeBlob() with BLOBEscapingType::ZeroXHex conversion type is used. */
+KDB_EXPORT KDbEscapedString valueToSql(KDbField::Type ftype, const QVariant& v);
 
 /*! Converts value @a v to string representation required by KDbSQL commands:
     ISO 8601 DateTime format - with "T" delimiter/
     For specification see http://www.w3.org/TR/NOTE-datetime.
     Example: "1994-11-05T13:15:30" not "1994-11-05 13:15:30".
     @todo Add support for time zones */
-KDB_EXPORT KDbEscapedString dateTimeToSQL(const QDateTime& v);
+KDB_EXPORT KDbEscapedString dateTimeToSql(const QDateTime& v);
 
 #ifdef KDB_DEBUG_GUI
 //! A prototype of handler for GUI debugger
@@ -693,8 +738,8 @@ KDB_EXPORT QStringList libraryPaths();
  the returned name as quickly as possible for creating new physical table.
  It is not 100% guaranteed that table with this name will not exist at an attempt of creation
  but it is very unlikely. The function checks for existence of a table with temporary name
- for connection @a conn. Empty string is returned if @a conn is not present or is not open
- or if checking for existence of table withg temporary name failed.
+ for connection @a conn. Empty string is returned if @a conn is @c nullptr or is not open
+ or if checking for existence of table with temporary name failed.
 */
 KDB_EXPORT QString temporaryTableName(KDbConnection *conn, const QString &baseName);
 
@@ -708,10 +753,14 @@ KDB_EXPORT QString sqlite3ProgramPath();
  @return true on success. */
 KDB_EXPORT bool importSqliteFile(const QString &inputFileName, const QString &outputFileName);
 
-/*! @return true if @a s is a valid identifier, ie. starts with a letter or '_' character
+/*! @return @c true if @a s is a valid identifier, i.e. starts with a letter or '_' character
  and contains only letters, numbers and '_' character. */
 KDB_EXPORT bool isIdentifier(const QString& s);
 
+//! @overload isIdentifier(const QString& s)
+//! @since 3.1
+KDB_EXPORT bool isIdentifier(const QByteArray& s);
+
 /*! @return valid identifier based on @a s.
  Non-alphanumeric characters (or spaces) are replaced with '_'.
  If a number is at the beginning, '_' is added at start.
