@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2003-2016 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2003-2017 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -19,40 +19,43 @@
 
 #include "KDbNativeStatementBuilder.h"
 #include "KDbConnection.h"
+#include "kdb_debug.h"
 #include "KDbDriverBehavior.h"
+#include "KDbDriver_p.h"
 #include "KDbExpression.h"
 #include "KDbLookupFieldSchema.h"
+#include "KDbOrderByColumn.h"
+#include "KDbQueryAsterisk.h"
 #include "KDbQuerySchema.h"
 #include "KDbQuerySchemaParameter.h"
 #include "KDbRelationship.h"
-#include "kdb_debug.h"
 
-KDbSelectStatementOptions::KDbSelectStatementOptions()
-        : alsoRetrieveRecordId(false)
-        , addVisibleLookupColumns(true)
+KDbSelectStatementOptions::~KDbSelectStatementOptions()
 {
 }
 
 //================================================
 
-class KDbNativeStatementBuilder::Private
+class Q_DECL_HIDDEN KDbNativeStatementBuilder::Private
 {
 public:
     Private() {}
     //! @todo use equivalent of QPointer<KDbConnection>
     KDbConnection *connection;
+    KDb::IdentifierEscapingType dialect;
 
-    inline KDbDriver *driver() { return connection ? connection->driver() : nullptr; }
 private:
     Q_DISABLE_COPY(Private)
 };
 
 //================================================
 
-KDbNativeStatementBuilder::KDbNativeStatementBuilder(KDbConnection *connection)
+KDbNativeStatementBuilder::KDbNativeStatementBuilder(KDbConnection *connection,
+                                                     KDb::IdentifierEscapingType dialect)
     : d(new Private)
 {
     d->connection = connection;
+    d->dialect = dialect;
 }
 
 KDbNativeStatementBuilder::~KDbNativeStatementBuilder()
@@ -62,6 +65,7 @@ KDbNativeStatementBuilder::~KDbNativeStatementBuilder()
 
 static bool selectStatementInternal(KDbEscapedString *target,
                                     KDbConnection *connection,
+                                    KDb::IdentifierEscapingType dialect,
                                     KDbQuerySchema* querySchema,
                                     const KDbSelectStatementOptions& options,
                                     const QList<QVariant>& parameters)
@@ -73,7 +77,7 @@ static bool selectStatementInternal(KDbEscapedString *target,
 //  return QString();
 // Each SQL identifier needs to be escaped in the generated query.
 
-    const KDbDriver *driver = connection ? connection->driver() : 0;
+    const KDbDriver *driver = dialect == KDb::DriverEscaping ? connection->driver() : nullptr;
 
     if (!querySchema->statement().isEmpty()) {
 //! @todo replace with KDbNativeQuerySchema? It shouldn't be here.
@@ -84,7 +88,8 @@ static bool selectStatementInternal(KDbEscapedString *target,
 //! @todo looking at singleTable is visually nice but a field name can conflict
 //!   with function or variable name...
     int number = 0;
-    bool singleTable = querySchema->tables()->count() <= 1;
+    QList<KDbTableSchema*>* tables = querySchema->tables();
+    bool singleTable = tables->count() <= 1;
     if (singleTable) {
         //make sure we will have single table:
         foreach(KDbField *f, *querySchema->fields()) {
@@ -105,22 +110,42 @@ static bool selectStatementInternal(KDbEscapedString *target,
     int internalUniqueQueryAliasNumber = 0; //used to build internalUniqueQueryAliases
     number = 0;
     QList<KDbQuerySchema*> subqueries_for_lookup_data; // subqueries will be added to FROM section
-    KDbEscapedString kdb_subquery_prefix("__kdb_subquery_");
+    const QString kdb_subquery_prefix = QStringLiteral("__kdb_subquery_");
+    KDbQuerySchemaParameterValueListIterator paramValuesIt(parameters);
+    KDbQuerySchemaParameterValueListIterator *paramValuesItPtr
+        = parameters.isEmpty() ? nullptr : &paramValuesIt;
     foreach(KDbField *f, *querySchema->fields()) {
         if (querySchema->isColumnVisible(number)) {
             if (!sql.isEmpty())
                 sql += ", ";
 
             if (f->isQueryAsterisk()) {
-                if (!singleTable && static_cast<KDbQueryAsterisk*>(f)->isSingleTableAsterisk()) { //single-table *
-                    sql.append(KDb::escapeIdentifier(driver, f->table()->name())).append(".*");
-                }
-                else { //all-tables * (or simplified table.* when there's only one table)
-                    sql += '*';
+                KDbQueryAsterisk *asterisk = static_cast<KDbQueryAsterisk*>(f);
+                if (!singleTable && asterisk->isSingleTableAsterisk()) { //single-table *
+                    sql.append(KDb::escapeIdentifier(driver, asterisk->table()->name())).append(".*");
+                } else {
+                    /* All-tables asterisk
+                     NOTE: do not output in this form because there can be extra tables
+                     automatically added for obtaining lookup data what changes number of fields.
+                     Reliable solution to that: for tables T1..Tn output T1.*,..Tn.*
+                     Example for Northwind:
+                     - instead of: SELECT * FROM orders LEFT OUTER JOIN
+                                   customers ON orders.customerid=customers.customerid
+                     - use this: SELECT orders.*, customers.contactname FROM orders LEFT OUTER JOIN
+                                 customers ON orders.customerid=customers.customerid
+                    */
+                    KDbEscapedString s_tables;
+                    for (KDbTableSchema *table : *tables) {
+                        if (!s_tables.isEmpty()) {
+                            s_tables += ", ";
+                        }
+                        s_tables.append(KDb::escapeIdentifier(driver, table->name()) + QLatin1String(".*"));
+                    }
+                    sql += s_tables;
                 }
             } else {
                 if (f->isExpression()) {
-                    sql += f->expression().toString(driver);
+                    sql += f->expression().toString(driver, paramValuesItPtr);
                 } else {
                     if (!f->table()) {//sanity check
                         return false;
@@ -132,7 +157,7 @@ static bool selectStatementInternal(KDbEscapedString *target,
                         tableName = KDb::iifNotEmpty(querySchema->tableAlias(tablePosition),
                                                            f->table()->name());
                     }
-                    if (options.addVisibleLookupColumns) { // try to find table/alias name harder
+                    if (options.addVisibleLookupColumns()) { // try to find table/alias name harder
                         if (tableName.isEmpty()) {
                             tableName = querySchema->tableAlias(f->table()->name());
                         }
@@ -151,18 +176,18 @@ static bool selectStatementInternal(KDbEscapedString *target,
                 }
 //! @todo add option that allows to omit "AS" keyword
             }
-            KDbLookupFieldSchema *lookupFieldSchema = (options.addVisibleLookupColumns && f->table())
-                                                   ? f->table()->lookupFieldSchema(*f) : 0;
+            KDbLookupFieldSchema *lookupFieldSchema = (options.addVisibleLookupColumns() && f->table())
+                                                   ? f->table()->lookupFieldSchema(*f) : nullptr;
             if (lookupFieldSchema && lookupFieldSchema->boundColumn() >= 0) {
                 // Lookup field schema found
                 // Now we also need to fetch "visible" value from the lookup table, not only the value of binding.
                 // -> build LEFT OUTER JOIN clause for this purpose (LEFT, not INNER because the binding can be broken)
                 // "LEFT OUTER JOIN lookupTable ON thisTable.thisField=lookupTable.boundField"
-                KDbLookupFieldSchema::RecordSource recordSource = lookupFieldSchema->recordSource();
-                if (recordSource.type() == KDbLookupFieldSchema::RecordSource::Table) {
-                    KDbTableSchema *lookupTable = querySchema->connection()->tableSchema(recordSource.name());
-                    KDbFieldList* visibleColumns = 0;
-                    KDbField *boundField = 0;
+                KDbLookupFieldSchemaRecordSource recordSource = lookupFieldSchema->recordSource();
+                if (recordSource.type() == KDbLookupFieldSchemaRecordSource::Type::Table) {
+                    KDbTableSchema *lookupTable = connection->tableSchema(recordSource.name());
+                    KDbFieldList* visibleColumns = nullptr;
+                    KDbField *boundField = nullptr;
                     if (lookupTable
                             && lookupFieldSchema->boundColumn() < lookupTable->fieldCount()
                             && (visibleColumns = lookupTable->subList(lookupFieldSchema->visibleColumns()))
@@ -188,16 +213,17 @@ static bool selectStatementInternal(KDbEscapedString *target,
 //! @todo Add possibility for joining the values at client side.
                         s_additional_fields += visibleColumns->sqlFieldsList(
                                                    connection, QLatin1String(" || ' ' || "), internalUniqueTableAlias,
-                                                   driver ? KDb::DriverEscaping : KDb::KDbEscaping);
+                                                   dialect);
                     }
                     delete visibleColumns;
-                } else if (recordSource.type() == KDbLookupFieldSchema::RecordSource::Query) {
-                    KDbQuerySchema *lookupQuery = querySchema->connection()->querySchema(recordSource.name());
+                } else if (recordSource.type() == KDbLookupFieldSchemaRecordSource::Type::Query) {
+                    KDbQuerySchema *lookupQuery = connection->querySchema(recordSource.name());
                     if (!lookupQuery) {
                         kdbWarning() << "!lookupQuery";
                         return false;
                     }
-                    const KDbQueryColumnInfo::Vector fieldsExpanded(lookupQuery->fieldsExpanded());
+                    const KDbQueryColumnInfo::Vector fieldsExpanded(
+                        lookupQuery->fieldsExpanded(connection));
                     if (lookupFieldSchema->boundColumn() >= fieldsExpanded.count()) {
                         kdbWarning() << "lookupFieldSchema->boundColumn() >= fieldsExpanded.count()";
                         return false;
@@ -207,7 +233,7 @@ static bool selectStatementInternal(KDbEscapedString *target,
                         kdbWarning() << "!boundColumnInfo";
                         return false;
                     }
-                    KDbField *boundField = boundColumnInfo->field;
+                    KDbField *boundField = boundColumnInfo->field();
                     if (!boundField) {
                         kdbWarning() << "!boundField";
                         return false;
@@ -215,10 +241,11 @@ static bool selectStatementInternal(KDbEscapedString *target,
                     //add LEFT OUTER JOIN
                     if (!s_additional_joins.isEmpty())
                         s_additional_joins += ' ';
-                    KDbEscapedString internalUniqueQueryAlias
-                        = kdb_subquery_prefix + connection->escapeString(lookupQuery->name()) + '_'
-                        + QString::number(internalUniqueQueryAliasNumber++);
-                    KDbNativeStatementBuilder builder(connection);
+                    KDbEscapedString internalUniqueQueryAlias(KDb::escapeIdentifier(
+                        driver,
+                        kdb_subquery_prefix + lookupQuery->name() + QLatin1Char('_')
+                            + QString::number(internalUniqueQueryAliasNumber++)));
+                    KDbNativeStatementBuilder builder(connection, dialect);
                     KDbEscapedString subSql;
                     if (!builder.generateSelectStatement(&subSql, lookupQuery, options,
                                                          parameters))
@@ -267,32 +294,34 @@ static bool selectStatementInternal(KDbEscapedString *target,
     if (!s_additional_fields.isEmpty())
         sql += (", " + s_additional_fields);
 
-    if (driver && options.alsoRetrieveRecordId) { //append rowid column
+    if (driver && options.alsoRetrieveRecordId()) { //append rowid column
+        //! @todo Check if the rowid isn't already part of regular SELECT columns, if so, don't add
         KDbEscapedString s;
         if (!sql.isEmpty())
             s = ", ";
         if (querySchema->masterTable())
             s += KDbEscapedString(querySchema->tableAliasOrName(querySchema->masterTable()->name())) + '.';
-        s += driver->behavior()->ROW_ID_FIELD_NAME;
+        s += KDbDriverPrivate::behavior(driver)->ROW_ID_FIELD_NAME;
         sql += s;
     }
 
-    sql.prepend("SELECT ");
-    QList<KDbTableSchema*>* tables = querySchema->tables();
-    if ((tables && !tables->isEmpty()) || !subqueries_for_lookup_data.isEmpty()) {
+    if (sql.isEmpty()) {
+        sql.prepend("SELECT"); // "SELECT FROM ..." case
+    } else {
+        sql.prepend("SELECT ");
+    }
+    if (!tables->isEmpty() || !subqueries_for_lookup_data.isEmpty()) {
         sql += " FROM ";
         KDbEscapedString s_from;
-        if (tables) {
-            number = 0;
-            foreach(KDbTableSchema *table, *tables) {
-                if (!s_from.isEmpty())
-                    s_from += ", ";
-                s_from += KDb::escapeIdentifier(driver, table->name());
-                const QString aliasString(querySchema->tableAlias(number));
-                if (!aliasString.isEmpty())
-                    s_from.append(" AS ").append(aliasString);
-                number++;
-            }
+        number = 0;
+        foreach(KDbTableSchema *table, *tables) {
+            if (!s_from.isEmpty())
+                s_from += ", ";
+            s_from += KDb::escapeIdentifier(driver, table->name());
+            const QString aliasString(querySchema->tableAlias(number));
+            if (!aliasString.isEmpty())
+                s_from.append(" AS ").append(aliasString);
+            number++;
         }
         // add subqueries for lookup data
         int subqueries_for_lookup_data_counter = 0;
@@ -300,11 +329,13 @@ static bool selectStatementInternal(KDbEscapedString *target,
             if (!s_from.isEmpty())
                 s_from += ", ";
             KDbEscapedString subSql;
-            if (!selectStatementInternal(&subSql, connection, subQuery, options, parameters)) {
+            if (!selectStatementInternal(&subSql, connection, dialect, subQuery, options, parameters)) {
                 return false;
             }
-            s_from += '(' + subSql + ") AS " + kdb_subquery_prefix
-                      + KDbEscapedString::number(subqueries_for_lookup_data_counter++);
+            s_from += '(' + subSql + ") AS "
+                + KDb::escapeIdentifier(
+                      driver,
+                      kdb_subquery_prefix + QString::number(subqueries_for_lookup_data_counter++));
         }
         sql += s_from;
     }
@@ -342,11 +373,9 @@ static bool selectStatementInternal(KDbEscapedString *target,
         s_where += s_where_sub;
     }
     //EXPLICITLY SPECIFIED WHERE EXPRESSION
-    if (driver && !querySchema->whereExpression().isNull()) {
-        KDbQuerySchemaParameterValueListIterator paramValuesIt(parameters);
-        KDbQuerySchemaParameterValueListIterator *paramValuesItPtr = parameters.isEmpty() ? 0 : &paramValuesIt;
+    if (!querySchema->whereExpression().isNull()) {
         if (wasWhere) {
-//! @todo () are not always needed
+            //! @todo () are not always needed
             s_where = '(' + s_where + ") AND ("
                 + querySchema->whereExpression().toString(driver, paramValuesItPtr) + ')';
         } else {
@@ -359,15 +388,14 @@ static bool selectStatementInternal(KDbEscapedString *target,
     //(use wasWhere here)
 
     // ORDER BY
-    KDbEscapedString orderByString(
-        querySchema->orderByColumnList()->toSQLString(
-            !singleTable/*includeTableName*/, connection, driver ? KDb::DriverEscaping : KDb::KDbEscaping)
-    );
-    const QVector<int> pkeyFieldsOrder(querySchema->pkeyFieldsOrder());
-    if (orderByString.isEmpty() && !pkeyFieldsOrder.isEmpty()) {
-        //add automatic ORDER BY if there is no explicitly defined (especially helps when there are complex JOINs)
+    KDbEscapedString orderByString(querySchema->orderByColumnList()->toSqlString(
+        !singleTable /*includeTableName*/, connection, dialect));
+    const QVector<int> pkeyFieldsOrder(querySchema->pkeyFieldsOrder(connection));
+    if (dialect == KDb::DriverEscaping  && orderByString.isEmpty() && !pkeyFieldsOrder.isEmpty()) {
+        // Native only: add automatic ORDER BY if there is no explicitly defined one
+        // (especially helps when there are complex JOINs)
         KDbOrderByColumnList automaticPKOrderBy;
-        const KDbQueryColumnInfo::Vector fieldsExpanded(querySchema->fieldsExpanded());
+        const KDbQueryColumnInfo::Vector fieldsExpanded(querySchema->fieldsExpanded(connection));
         foreach(int pkeyFieldsIndex, pkeyFieldsOrder) {
             if (pkeyFieldsIndex < 0) // no field mentioned in this query
                 continue;
@@ -379,8 +407,8 @@ static bool selectStatementInternal(KDbEscapedString *target,
             KDbQueryColumnInfo *ci = fieldsExpanded[ pkeyFieldsIndex ];
             automaticPKOrderBy.appendColumn(ci);
         }
-        orderByString = automaticPKOrderBy.toSQLString(!singleTable/*includeTableName*/,
-                        connection, driver ? KDb::DriverEscaping : KDb::KDbEscaping);
+        orderByString = automaticPKOrderBy.toSqlString(!singleTable /*includeTableName*/,
+                                                       connection, dialect);
     }
     if (!orderByString.isEmpty())
         sql += (" ORDER BY " + orderByString);
@@ -395,14 +423,14 @@ bool KDbNativeStatementBuilder::generateSelectStatement(KDbEscapedString *target
                                                         const KDbSelectStatementOptions& options,
                                                         const QList<QVariant>& parameters) const
 {
-    return selectStatementInternal(target, d->connection, querySchema, options, parameters);
+    return selectStatementInternal(target, d->connection, d->dialect, querySchema, options, parameters);
 }
 
 bool KDbNativeStatementBuilder::generateSelectStatement(KDbEscapedString *target,
                                                         KDbQuerySchema* querySchema,
                                                         const QList<QVariant>& parameters) const
 {
-    return selectStatementInternal(target, d->connection, querySchema, KDbSelectStatementOptions(),
+    return selectStatementInternal(target, d->connection, d->dialect, querySchema, KDbSelectStatementOptions(),
                                    parameters);
 }
 
@@ -416,39 +444,41 @@ bool KDbNativeStatementBuilder::generateSelectStatement(KDbEscapedString *target
 bool KDbNativeStatementBuilder::generateCreateTableStatement(KDbEscapedString *target,
                                                              const KDbTableSchema& tableSchema) const
 {
-    Q_ASSERT(target);
+    if (!target) {
+        return false;
+    }
     // Each SQL identifier needs to be escaped in the generated query.
-    const KDbDriver *driver = d->connection ? d->connection->driver() : nullptr;
+    const KDbDriver *driver = d->dialect == KDb::DriverEscaping ? d->connection->driver() : nullptr;
     KDbEscapedString sql;
     sql.reserve(4096);
     sql = KDbEscapedString("CREATE TABLE ")
             + KDb::escapeIdentifier(driver, tableSchema.name()) + " (";
     bool first = true;
-    foreach(KDbField *field, tableSchema.m_fields) {
+    for (const KDbField *field : *tableSchema.fields()) {
         if (first)
             first = false;
         else
             sql += ", ";
         KDbEscapedString v = KDbEscapedString(KDb::escapeIdentifier(driver, field->name())) + ' ';
         const bool autoinc = field->isAutoIncrement();
-        const bool pk = field->isPrimaryKey() || (autoinc && driver && driver->beh->AUTO_INCREMENT_REQUIRES_PK);
+        const bool pk = field->isPrimaryKey() || (autoinc && driver && driver->behavior()->AUTO_INCREMENT_REQUIRES_PK);
 //! @todo warning: ^^^^^ this allows only one autonumber per table when AUTO_INCREMENT_REQUIRES_PK==true!
         const KDbField::Type type = field->type(); // cache: evaluating type of expressions can be expensive
-        if (autoinc && d->driver()->beh->SPECIAL_AUTO_INCREMENT_DEF) {
+        if (autoinc && d->connection->driver()->behavior()->SPECIAL_AUTO_INCREMENT_DEF) {
             if (pk)
-                v.append(d->driver()->beh->AUTO_INCREMENT_TYPE).append(' ')
-                 .append(d->driver()->beh->AUTO_INCREMENT_PK_FIELD_OPTION);
+                v.append(d->connection->driver()->behavior()->AUTO_INCREMENT_TYPE).append(' ')
+                 .append(d->connection->driver()->behavior()->AUTO_INCREMENT_PK_FIELD_OPTION);
             else
-                v.append(d->driver()->beh->AUTO_INCREMENT_TYPE).append(' ')
-                 .append(d->driver()->beh->AUTO_INCREMENT_FIELD_OPTION);
+                v.append(d->connection->driver()->behavior()->AUTO_INCREMENT_TYPE).append(' ')
+                 .append(d->connection->driver()->behavior()->AUTO_INCREMENT_FIELD_OPTION);
         } else {
-            if (autoinc && !d->driver()->beh->AUTO_INCREMENT_TYPE.isEmpty())
-                v += d->driver()->beh->AUTO_INCREMENT_TYPE;
+            if (autoinc && !d->connection->driver()->behavior()->AUTO_INCREMENT_TYPE.isEmpty())
+                v += d->connection->driver()->behavior()->AUTO_INCREMENT_TYPE;
             else
-                v += d->driver()->sqlTypeName(type, *field);
+                v += d->connection->driver()->sqlTypeName(type, *field);
 
             if (KDbField::isIntegerType(type) && field->isUnsigned()) {
-                v.append(' ').append(d->driver()->beh->UNSIGNED_TYPE_KEYWORD);
+                v.append(' ').append(d->connection->driver()->behavior()->UNSIGNED_TYPE_KEYWORD);
             }
 
             if (KDbField::isFPNumericType(type) && field->precision() > 0) {
@@ -459,15 +489,15 @@ bool KDbNativeStatementBuilder::generateCreateTableStatement(KDbEscapedString *t
             }
             else if (type == KDbField::Text) {
                 int realMaxLen;
-                if (d->driver()->beh->TEXT_TYPE_MAX_LENGTH == 0) {
+                if (d->connection->driver()->behavior()->TEXT_TYPE_MAX_LENGTH == 0) {
                     realMaxLen = field->maxLength(); // allow to skip (N)
                 }
                 else { // max length specified by driver
                     if (field->maxLength() == 0) { // as long as possible
-                        realMaxLen = d->driver()->beh->TEXT_TYPE_MAX_LENGTH;
+                        realMaxLen = d->connection->driver()->behavior()->TEXT_TYPE_MAX_LENGTH;
                     }
                     else { // not longer than specified by driver
-                        realMaxLen = qMin(d->driver()->beh->TEXT_TYPE_MAX_LENGTH, field->maxLength());
+                        realMaxLen = qMin(d->connection->driver()->behavior()->TEXT_TYPE_MAX_LENGTH, field->maxLength());
                     }
                 }
                 if (realMaxLen > 0) {
@@ -476,8 +506,8 @@ bool KDbNativeStatementBuilder::generateCreateTableStatement(KDbEscapedString *t
             }
 
             if (autoinc) {
-                v.append(' ').append(pk ? d->driver()->beh->AUTO_INCREMENT_PK_FIELD_OPTION
-                                        : d->driver()->beh->AUTO_INCREMENT_FIELD_OPTION);
+                v.append(' ').append(pk ? d->connection->driver()->behavior()->AUTO_INCREMENT_PK_FIELD_OPTION
+                                        : d->connection->driver()->behavior()->AUTO_INCREMENT_FIELD_OPTION);
             }
             else {
                 //! @todo here is automatically a single-field key created
@@ -489,10 +519,10 @@ bool KDbNativeStatementBuilder::generateCreateTableStatement(KDbEscapedString *t
 ///@todo IS this ok for all engines?: if (!autoinc && !field->isPrimaryKey() && field->isNotNull())
             if (!autoinc && !pk && field->isNotNull())
                 v += " NOT NULL"; //only add not null option if no autocommit is set
-            if (d->driver()->supportsDefaultValue(*field) && field->defaultValue().isValid()) {
-                KDbEscapedString valToSQL(d->driver()->valueToSQL(field, field->defaultValue()));
-                if (!valToSQL.isEmpty()) //for sanity
-                    v += " DEFAULT " + valToSQL;
+            if (d->connection->driver()->supportsDefaultValue(*field) && field->defaultValue().isValid()) {
+                KDbEscapedString valToSql(d->connection->driver()->valueToSql(field, field->defaultValue()));
+                if (!valToSql.isEmpty()) //for sanity
+                    v += " DEFAULT " + valToSql;
             }
         }
         sql += v;
