@@ -29,22 +29,22 @@
 #include "KDbRecordData.h"
 
 // Constructor based on query statement
-PostgresqlCursor::PostgresqlCursor(KDbConnection* conn, const KDbEscapedString& sql, int options)
-        : KDbCursor(conn, sql, options)
+PostgresqlCursor::PostgresqlCursor(KDbConnection* conn, const KDbEscapedString& sql,
+                                   KDbCursor::Options options)
+        : KDbCursor(conn, sql, options | KDbCursor::Option::Buffered)
         , m_numRows(0)
         , d(new PostgresqlCursorData(conn))
 {
-    m_options |= Buffered;
 }
 
 //==================================================================================
 //Constructor base on query object
-PostgresqlCursor::PostgresqlCursor(KDbConnection* conn, KDbQuerySchema* query, int options)
-        : KDbCursor(conn, query, options)
+PostgresqlCursor::PostgresqlCursor(KDbConnection* conn, KDbQuerySchema* query,
+                                   KDbCursor::Options options)
+        : KDbCursor(conn, query, options | KDbCursor::Option::Buffered)
         , m_numRows(0)
         , d(new PostgresqlCursorData(conn))
 {
-    m_options |= Buffered;
 }
 
 //==================================================================================
@@ -59,7 +59,7 @@ PostgresqlCursor::~PostgresqlCursor()
 //Create a cursor result set
 bool PostgresqlCursor::drv_open(const KDbEscapedString& sql)
 {
-    d->res = d->executeSQL(sql);
+    d->res = d->executeSql(sql);
     d->resultStatus = PQresultStatus(d->res);
     if (d->resultStatus != PGRES_TUPLES_OK && d->resultStatus != PGRES_COMMAND_OK) {
         storeResultAndClear(&d->res, d->resultStatus);
@@ -97,15 +97,15 @@ bool PostgresqlCursor::drv_close()
 void PostgresqlCursor::drv_getNextRecord()
 {
     if (at() >= qint64(m_numRows)) {
-        m_fetchResult = FetchEnd;
+        m_fetchResult = FetchResult::End;
     }
     else if (at() < 0) {
         // control will reach here only when at() < 0 ( which is usually -1 )
         // -1 is same as "1 beyond the End"
-        m_fetchResult = FetchEnd;
+        m_fetchResult = FetchResult::End;
     }
     else { // 0 <= at() < m_numRows
-        m_fetchResult = FetchOK;
+        m_fetchResult = FetchResult::Ok;
     }
 }
 
@@ -115,11 +115,11 @@ void PostgresqlCursor::drv_getNextRecord()
 void PostgresqlCursor::drv_getPrevRecord()
 {
     if (at() < m_res->size() && at() >= 0) {
-        m_fetchResult = FetchOK;
+        m_fetchResult = FetchResult::Ok;
     } else if (at() >= m_res->size()) {
-        m_fetchResult = FetchEnd;
+        m_fetchResult = FetchResult::End;
     } else {
-        m_fetchResult = FetchError;
+        m_fetchResult = FetchResult::Error;
     }
 }
 #endif
@@ -226,7 +226,7 @@ QVariant PostgresqlCursor::pValue(int pos) const
     const qint64 row = at();
 
     KDbField *f = (m_visibleFieldsExpanded && pos < qMin(m_visibleFieldsExpanded->count(), m_fieldCount))
-                       ? m_visibleFieldsExpanded->at(pos)->field : 0;
+                       ? m_visibleFieldsExpanded->at(pos)->field() : nullptr;
 // postgresqlDebug() << "pos:" << pos;
 
     const KDbField::Type type = m_realTypes[pos];
@@ -294,7 +294,7 @@ QVariant PostgresqlCursor::pValue(int pos) const
 const char** PostgresqlCursor::recordData() const
 {
     //! @todo
-    return 0;
+    return nullptr;
 }
 
 //==================================================================================
