@@ -29,9 +29,9 @@
 
 KDbVariableExpressionData::KDbVariableExpressionData()
  : KDbExpressionData()
- , field(0)
+ , field(nullptr)
  , tablePositionForField(-1)
- , tableForQueryAsterisk(0)
+ , tableForQueryAsterisk(nullptr)
 {
     ExpressionDebug << "VariableExpressionData" << ref;
 }
@@ -39,9 +39,9 @@ KDbVariableExpressionData::KDbVariableExpressionData()
 KDbVariableExpressionData::KDbVariableExpressionData(const QString& aName)
  : KDbExpressionData()
  , name(aName)
- , field(0)
+ , field(nullptr)
  , tablePositionForField(-1)
- , tableForQueryAsterisk(0)
+ , tableForQueryAsterisk(nullptr)
 {
    ExpressionDebug << "VariableExpressionData" << ref;
 }
@@ -61,7 +61,7 @@ void KDbVariableExpressionData::debugInternal(QDebug dbg, KDb::ExpressionCallSta
 {
     Q_UNUSED(callStack);
     dbg.nospace() << qPrintable(QString::fromLatin1("VariableExp(\"%1\",type=%2)")
-        .arg(name, field ? KDbDriver::defaultSQLTypeName(type())
+        .arg(name, field ? KDbDriver::defaultSqlTypeName(type())
                              : QLatin1String("FIELD_NOT_DEFINED_YET")));
 }
 
@@ -103,9 +103,9 @@ bool KDbVariableExpressionData::validateInternal(KDbParseInfo *parseInfo_, KDb::
 {
     Q_UNUSED(callStack);
     KDbParseInfoInternal *parseInfo = static_cast<KDbParseInfoInternal*>(parseInfo_);
-    field = 0;
+    field = nullptr;
     tablePositionForField = -1;
-    tableForQueryAsterisk = 0;
+    tableForQueryAsterisk = nullptr;
 
     /* taken from parser's addColumn(): */
     kdbDebug() << "checking variable name: " << name;
@@ -122,7 +122,7 @@ bool KDbVariableExpressionData::validateInternal(KDbParseInfo *parseInfo_, KDb::
         }
 
         //find first table that has this field
-        KDbField *firstField = 0;
+        KDbField *firstField = nullptr;
         foreach(KDbTableSchema *table, *parseInfo->querySchema()->tables()) {
             KDbField *f = table->field(fieldName);
             if (f) {
@@ -198,12 +198,12 @@ bool KDbVariableExpressionData::validateInternal(KDbParseInfo *parseInfo_, KDb::
         return false;
     }
 
-    if (parseInfo->tablesAndAliasesForName(tableName).isEmpty()) {  //for sanity
+    const QList<int> positionsList(parseInfo->tablesAndAliasesForName(tableName));
+    if (positionsList.isEmpty()) {  //for sanity
         validateImplError(parseInfo,
             QString::fromLatin1("%1.%2, !positionsList ").arg(tableName, fieldName));
         return false;
     }
-    const QList<int> positionsList(parseInfo->tablesAndAliasesForName(tableName));
 
     //it's a table.*
     if (fieldName == QLatin1String("*")) {
